@@ -58,11 +58,29 @@ static void
 control_connection_died_cb (gpointer connection,
 			    gpointer user_data)
 {
+	CORBA_Object cntl;
+	CORBA_Environment ev;
 	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (user_data);
 
 	g_return_if_fail (frame != NULL);
 
 	dprintf ("The remote control end died unexpectedly");
+
+	CORBA_exception_init (&ev);
+
+	cntl = CORBA_Object_duplicate (frame->priv->control, &ev);
+
+	bonobo_control_frame_bind_to_control (
+		frame, CORBA_OBJECT_NIL, NULL);
+
+	CORBA_exception_set_system (
+		&ev, ex_CORBA_COMM_FAILURE, CORBA_COMPLETED_YES);
+
+	bonobo_object_check_env (BONOBO_OBJECT (frame), cntl, &ev);
+
+	CORBA_Object_release (cntl, &ev);
+
+	CORBA_exception_free (&ev);
 }
 
 static Bonobo_Gdk_WindowId
@@ -83,12 +101,24 @@ impl_Bonobo_ControlFrame_getToplevelId (PortableServer_Servant  servant,
 	bonobo_return_val_if_fail (toplev != NULL, NULL, ev);
 
 	if (BONOBO_IS_PLUG (toplev)) { 
-		BonoboControl     *control;
+		BonoboControl      *control;
+		Bonobo_ControlFrame frame;
 
 		control = bonobo_plug_get_control (BONOBO_PLUG (toplev));
+		if (!control) {
+			g_warning ("No control bound to plug from which to "
+				   "get transient parent");
+			return CORBA_string_dup ("");
+		}
 
-		id = Bonobo_ControlFrame_getToplevelId (
-			BONOBO_OBJREF (control), ev);
+		frame = bonobo_control_get_control_frame (control, ev);
+		if (frame == CORBA_OBJECT_NIL) {
+			g_warning ("No control frame associated with control from "
+				   "which to get transient parent");
+			return CORBA_string_dup ("");
+		}
+
+		id = Bonobo_ControlFrame_getToplevelId (frame, ev);
 	} else
 		id = bonobo_control_window_id_from_x11 (
 			GDK_WINDOW_XWINDOW (toplev->window));
@@ -213,6 +243,32 @@ dump_gdk_tree (GdkWindow *window)
 }
 #endif
 
+static CORBA_char *
+bonobo_control_frame_get_remote_window_id (BonoboControlFrame *frame,
+					   CORBA_Environment  *ev)
+{
+#ifdef HAVE_GTK_MULTIHEAD
+	CORBA_char  *retval;
+	char        *cookie;
+	int          screen;
+
+	screen = gdk_screen_get_number (
+			gtk_widget_get_screen (frame->priv->socket));
+
+	cookie = g_strdup_printf ("screen=%d", screen);
+
+	retval = Bonobo_Control_getWindowId (
+				frame->priv->control, cookie, ev);
+
+	g_free (cookie);
+
+	return retval;
+#else
+	return Bonobo_Control_getWindowId (
+				frame->priv->control, "", ev);
+#endif /* HAVE_GTK_MULTIHEAD */
+}
+
 void
 bonobo_control_frame_get_remote_window (BonoboControlFrame *frame,
 					CORBA_Environment  *opt_ev)
@@ -240,8 +296,7 @@ bonobo_control_frame_get_remote_window (BonoboControlFrame *frame,
 		ev = opt_ev;
 
 	/* Introduce ourselves to the Control. */
-	id = Bonobo_Control_getWindowId (frame->priv->control, "", ev);
-
+	id = bonobo_control_frame_get_remote_window_id (frame, ev);
 	if (BONOBO_EX (ev)) {
 		dprintf ("getWindowId exception\n");
 		bonobo_object_check_env (BONOBO_OBJECT (frame),
@@ -904,7 +959,14 @@ bonobo_control_frame_size_request (BonoboControlFrame *frame,
 
 	g_return_if_fail (requisition != NULL);
 	g_return_if_fail (BONOBO_IS_CONTROL_FRAME (frame));
-	g_return_if_fail (frame->priv->control != CORBA_OBJECT_NIL);
+
+	if (frame->priv->control == CORBA_OBJECT_NIL) {
+		/* We haven't been bound to a control yet, so return "I don't
+		 * care about what I get assigned".
+		 */
+		requisition->width = requisition->height = 0;
+		return;
+	}
 
 	if (!opt_ev) {
 		CORBA_exception_init (&tmp_ev);
@@ -1016,10 +1078,9 @@ bonobo_control_frame_set_socket (BonoboControlFrame *frame,
 
 	old_socket = (BonoboSocket *) frame->priv->socket;
 
-	if (socket) {
+	if (socket)
 		frame->priv->socket = g_object_ref (socket);
-		bonobo_socket_set_control_frame (socket, frame);
-	} else
+	else
 		frame->priv->socket = NULL;
 
 	if (old_socket) {
@@ -1027,6 +1088,9 @@ bonobo_control_frame_set_socket (BonoboControlFrame *frame,
 			BONOBO_SOCKET (old_socket), NULL);
 		g_object_unref (old_socket);
 	}
+
+	if (socket)
+		bonobo_socket_set_control_frame (socket, frame);
 }
 
 BonoboSocket *
