@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-macros.h>
 #include <bonobo/bonobo-dock.h>
@@ -154,6 +155,8 @@ bonobo_dock_band_class_init (BonoboDockBandClass *class)
 static void
 bonobo_dock_band_instance_init (BonoboDockBand *band)
 {
+  GtkWidget *widget = GTK_WIDGET (band);
+
   GTK_WIDGET_SET_FLAGS (band, GTK_NO_WINDOW);
 
   band->_priv = NULL;
@@ -173,6 +176,9 @@ bonobo_dock_band_instance_init (BonoboDockBand *band)
   band->drag_allocation.width = band->drag_allocation.height = 0;
 
   band->new_for_drag = FALSE;
+
+  if (GTK_WIDGET_VISIBLE (widget))
+    gtk_widget_queue_resize (widget);
 }
 
 
@@ -1437,15 +1443,9 @@ GtkWidget *
 bonobo_dock_band_new (void)
 {
   BonoboDockBand *band;
-  GtkWidget *widget;
 
   band = g_object_new (bonobo_dock_band_get_type (), NULL);
-  widget = GTK_WIDGET (band);
-
-  if (GTK_WIDGET_VISIBLE (widget))
-    gtk_widget_queue_resize (widget);
-
-  return widget;
+  return GTK_WIDGET (band);
 }
 
 /**
@@ -1506,6 +1506,11 @@ bonobo_dock_band_insert (BonoboDockBand *band,
       && !docking_allowed (band, BONOBO_DOCK_ITEM (child)))
     return FALSE;
 
+  if (BONOBO_IS_DOCK_ITEM (child) &&
+      !bonobo_dock_item_set_orientation (BONOBO_DOCK_ITEM (child),
+					 band->orientation))
+      return FALSE;
+
   if (position < 0 || position > (gint) band->num_children)
     position = band->num_children;
 
@@ -1526,11 +1531,6 @@ bonobo_dock_band_insert (BonoboDockBand *band,
       g_list_prepend (p, band_child);
     }
 
-  if (BONOBO_IS_DOCK_ITEM (child)
-      && ! bonobo_dock_item_set_orientation (BONOBO_DOCK_ITEM (child),
-                                            band->orientation))
-      return FALSE;
-
   gtk_widget_set_parent (child, GTK_WIDGET (band));
 
   if (GTK_WIDGET_REALIZED (child->parent))
@@ -1907,3 +1907,75 @@ bonobo_dock_band_layout_add (BonoboDockBand *band,
                                     child_num, child->offset);
     }
 }
+
+static BonoboDock *
+get_dock (GtkWidget *widget)
+{
+	while (widget && !BONOBO_IS_DOCK (widget))
+		widget = widget->parent;
+
+	return (BonoboDock *) widget;
+}
+
+gint
+bonobo_dock_band_handle_key_nav (BonoboDockBand *band,
+				 BonoboDockItem *item,
+				 GdkEventKey    *event)
+{
+  gboolean handled = FALSE;
+
+  g_return_val_if_fail (BONOBO_IS_DOCK_BAND (band), FALSE);
+  g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (item), FALSE);
+
+  if (event->state & GDK_CONTROL_MASK)
+    {
+      GList *l;
+      int cur_idx = 0;
+      int dest_idx;
+
+      for (l = band->children; l; l = l->next)
+        {
+          BonoboDockBandChild *child = l->data;
+          if (child->widget == (GtkWidget *)item)
+            break;
+          cur_idx++;
+	}
+
+      g_return_val_if_fail (l != NULL, FALSE);
+      
+      dest_idx = cur_idx;
+      if (band->orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+	  if (event->keyval == GDK_Left)
+
+	      dest_idx--;
+	  if (event->keyval == GDK_Right)
+	      dest_idx++;
+	}
+      else
+        {
+	  if (event->keyval == GDK_Up)
+	      dest_idx--;
+	  if (event->keyval == GDK_Down)
+	      dest_idx++;
+	}
+
+      dest_idx = MAX (0, dest_idx);
+      dest_idx = MIN (g_list_length (band->children) - 1, dest_idx);
+      if (dest_idx != cur_idx)
+        {
+          handled = TRUE;
+	  bonobo_dock_band_move_child (band, l, dest_idx);
+	}
+  }
+
+  if (!handled)
+    {
+      BonoboDock *dock = get_dock (GTK_WIDGET (band));
+
+      if (dock)
+        handled = bonobo_dock_handle_key_nav (dock, band, item, event);
+    }
+
+  return handled;
+}
