@@ -69,21 +69,6 @@ enum {
 
 static guint signals [LAST_SIGNAL] = { 0 };
 
-struct _BonoboUIEnginePrivate {
-	GObject      *view;
-
-	BonoboUIXml  *tree;
-	int           frozen;
-	GSList       *syncs;
-	GSList       *state_updates;
-	GSList       *components;
-
-	BonoboUIContainer    *container;
-	BonoboUIEngineConfig *config;
-
-	GHashTable   *cmd_to_node;
-};
-
 /*
  *  Mapping from nodes to their synchronization
  * class information functions.
@@ -1426,18 +1411,19 @@ bonobo_ui_engine_set_ui_container (BonoboUIEngine    *engine,
 
 	old_container = engine->priv->container;
 
-	if (ui_container) {
+	if (ui_container)
 		engine->priv->container = BONOBO_UI_CONTAINER (
 			bonobo_object_ref (BONOBO_OBJECT (ui_container)));
-
-		bonobo_ui_container_set_engine (ui_container, engine);
-	} else
+	else
 		engine->priv->container = NULL;
 
 	if (old_container) {
 		bonobo_ui_container_set_engine (old_container, NULL);
 		bonobo_object_unref (BONOBO_OBJECT (old_container));
 	}
+
+	if (ui_container)
+		bonobo_ui_container_set_engine (ui_container, engine);
 }
 
 /**
@@ -1461,6 +1447,7 @@ real_exec_verb (BonoboUIEngine *engine,
 		const char     *component_name,
 		const char     *verb)
 {
+	char *verb_cpy;
 	Bonobo_UIComponent component;
 
 	g_return_if_fail (verb != NULL);
@@ -1471,13 +1458,17 @@ real_exec_verb (BonoboUIEngine *engine,
 
 	component = sub_component_objref (engine, component_name);
 
+	verb_cpy = g_strdup (verb);
+
 	if (component != CORBA_OBJECT_NIL) {
 		CORBA_Environment ev;
 
 		CORBA_exception_init (&ev);
 
+		CORBA_Object_duplicate (component, &ev);
+
 		Bonobo_UIComponent_execVerb (
-			component, verb, &ev);
+			component, verb_cpy, &ev);
 
 		if (engine->priv->container)
 			bonobo_object_check_env (
@@ -1485,14 +1476,16 @@ real_exec_verb (BonoboUIEngine *engine,
 				component, &ev);
 
 		if (BONOBO_EX (&ev))
-			g_warning ("Exception executing verb '%s' '%s'"
-				   "major %d, %s",
-				   verb, component_name, ev._major,
+			g_warning ("Exception executing verb '%s'"
+				   "major %d, %s", verb_cpy, ev._major,
 				   BONOBO_EX_REPOID (&ev));
+
+		CORBA_Object_release (component, &ev);
 		
 		CORBA_exception_free (&ev);
 	}
 
+	g_free (verb_cpy);
 	g_object_unref (engine);
 }
 
@@ -1738,7 +1731,7 @@ impl_emit_event_on (BonoboUIEngine *engine,
 {
 	const char      *id;
 	BonoboUIXmlData *data;
-	char            *component_id, *real_id;
+	char            *component_id, *real_id, *real_state;
 
 	g_return_if_fail (node != NULL);
 
@@ -1752,18 +1745,20 @@ impl_emit_event_on (BonoboUIEngine *engine,
 
 	component_id = g_strdup (data->id);
 	real_id      = g_strdup (id);
+	real_state   = g_strdup (state);
 
 	/* This could invoke a CORBA method that might de-register the component */
 	set_cmd_attr (engine, node, state_id, state, TRUE);
 
 	real_emit_ui_event (engine, component_id, real_id,
 			    Bonobo_UIComponent_STATE_CHANGED,
-			    state);
+			    real_state);
 
 	g_object_unref (engine);
 
-	g_free (component_id);
+	g_free (real_state);
 	g_free (real_id);
+	g_free (component_id);
 }
 
 void
@@ -1968,6 +1963,7 @@ BonoboUIEngine *
 bonobo_ui_engine_construct (BonoboUIEngine *engine,
 			    GObject        *view)
 {
+	GtkWindow *opt_parent;
 	BonoboUIEnginePrivate *priv;
 
 	g_return_val_if_fail (BONOBO_IS_UI_ENGINE (engine), NULL);
@@ -1978,8 +1974,14 @@ bonobo_ui_engine_construct (BonoboUIEngine *engine,
 	priv->tree = bonobo_ui_xml_new (
 		NULL, info_new_fn, info_free_fn,
 		info_dump_fn, add_node_fn, engine);
+
+
+	if (GTK_IS_WINDOW (view))
+		opt_parent = GTK_WINDOW (view);
+	else
+		opt_parent = NULL;
 	
-	priv->config = bonobo_ui_engine_config_new (engine);
+	priv->config = bonobo_ui_engine_config_new (engine, opt_parent);
 
 	build_skeleton (priv->tree);
 
