@@ -286,38 +286,6 @@ cmd_get_menu_pixbuf (GtkWidget    *widget,
 	return NULL;
 }
 
-/*
- * The second parameter uses underscores to escape keyboard
- * shortcuts. We ignore the first underscore, but if there
- * are two in a row, they are treated as if there was one.
- */
-static gboolean
-str_uscore_equal (const char *plain, const char *scored)
-{
-	while (*plain || *scored) {
-		if (*scored == '_') {
-			scored++;
-
-			/*
-			 * Must check for badly formed string
-			 * (ending with single underscore)
-			 * or risk reading off end.
-			 */
-			if (!*scored)
-				return FALSE;
-		}
-
-		if (*scored != *plain)
-			return FALSE;
-		else {
-			scored++;
-			plain++;
-		}
-	}
-
-	return TRUE;
-}
-
 static gboolean
 label_same (GtkBin *menu_widget, const char *txt)
 {
@@ -325,9 +293,9 @@ label_same (GtkBin *menu_widget, const char *txt)
 
 	return menu_widget &&
 		(label = menu_widget->child) &&
-		GTK_IS_ACCEL_LABEL (label) &&
+		GTK_IS_LABEL (label) &&
 		((GtkLabel *)label)->label &&
-		str_uscore_equal (((GtkLabel *)label)->label, txt);
+		!strcmp (((GtkLabel *)label)->label, txt);
 }
 
 static void
@@ -399,27 +367,32 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 	
 	if ((label_attr = bonobo_ui_engine_get_attr (node, cmd_node, "label"))) {
 		GtkWidget *label;
-
 		if (!label_same (GTK_BIN (menu_widget), label_attr)) {
-			label = gtk_accel_label_new (label_attr);
+			if (!GTK_BIN (menu_widget)->child) {
+				label = gtk_accel_label_new (label_attr);
 
-			/* this widget has a mnemonic */
-			gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+				g_object_freeze_notify (G_OBJECT (label));
+
+				/* this widget has a mnemonic */
+				gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+
+				/* Setup the widget. */
+				gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+				gtk_widget_show (label);
+				gtk_container_add (GTK_CONTAINER (menu_widget), label);
+
+			} else {
+				label = GTK_BIN (menu_widget)->child;
+
+				g_object_freeze_notify (G_OBJECT (label));
+
+				gtk_label_set_text_with_mnemonic (GTK_LABEL (label), label_attr);
+			}
 
-			/* Setup the widget. */
-			gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-			gtk_widget_show (label);
-			
-			/*
-			 * Insert it into the menu item widget and setup the
-			 * accelerator. FIXME: rather inefficient.
-			 */
-			if (GTK_BIN (menu_widget)->child)
-				gtk_widget_destroy (GTK_BIN (menu_widget)->child);
-			
-			gtk_container_add (GTK_CONTAINER (menu_widget), label);
 			gtk_accel_label_set_accel_widget (
 				GTK_ACCEL_LABEL (label), menu_widget);
+
+			g_object_thaw_notify (G_OBJECT (label));
 		} /* else
 			g_warning ("No change in label '%s'", label_attr); */
 
@@ -954,7 +927,7 @@ bonobo_ui_sync_menu_new (BonoboUIEngine *engine,
 		g_object_ref (menu_dock_item) :
 		menu_dock_item;
 
-	sync->accel_group = group ? g_object_ref (group) : NULL;
+	sync->accel_group = group ? g_object_ref (group) : gtk_accel_group_new ();
 
 	return bonobo_ui_sync_construct (
 		BONOBO_UI_SYNC (sync), engine, TRUE, TRUE);
