@@ -11,6 +11,7 @@
 #include <config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-macros.h>
+#include <bonobo/bonobo-ui-private.h>
 #include <bonobo/bonobo-ui-toolbar.h>
 #include <bonobo/bonobo-ui-toolbar-item.h>
 #include <bonobo/bonobo-ui-toolbar-popup-item.h>
@@ -92,22 +93,13 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static void
 parentize_widget (BonoboUIToolbar *toolbar,
-		  GtkWidget *widget)
+		  GtkWidget       *widget)
 {
 	g_assert (widget->parent == NULL);
 
 	/* The following is done according to the Bible, widget_system.txt, IV, 1.  */
 
 	gtk_widget_set_parent (widget, GTK_WIDGET (toolbar));
-
-	if (GTK_WIDGET_REALIZED (toolbar) && ! GTK_WIDGET_REALIZED (widget))
-		gtk_widget_realize (widget);
-
-	if (GTK_WIDGET_MAPPED (toolbar) && ! GTK_WIDGET_MAPPED (widget) &&  GTK_WIDGET_VISIBLE (widget))
-		gtk_widget_map (widget);
-
-	if (GTK_WIDGET_MAPPED (widget))
-		gtk_widget_queue_resize (GTK_WIDGET (toolbar));
 }
 
 static void
@@ -136,6 +128,9 @@ set_attributes_on_child (BonoboUIToolbarItem *item,
 	case BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY:
 		bonobo_ui_toolbar_item_set_style (item, BONOBO_UI_TOOLBAR_ITEM_STYLE_ICON_ONLY);
 		break;
+	case BONOBO_UI_TOOLBAR_STYLE_TEXT_ONLY:
+		bonobo_ui_toolbar_item_set_style (item, BONOBO_UI_TOOLBAR_ITEM_STYLE_TEXT_ONLY);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1047,7 +1042,7 @@ impl_set_property (GObject      *object,
 			toolbar, g_value_get_enum (value));
 		break;
 	case PROP_IS_FLOATING:
-		priv->is_floating = g_value_get_uint (value);
+		priv->is_floating = g_value_get_boolean (value);
 		break;
 	default:
 		break;
@@ -1073,6 +1068,7 @@ bonobo_ui_toolbar_class_init (BonoboUIToolbarClass *toolbar_class)
 	widget_class->map           = impl_map;
 	widget_class->unmap         = impl_unmap;
 	widget_class->expose_event  = impl_expose_event;
+	widget_class->focus         = bonobo_widget_clobber_focus;
 
 	container_class = GTK_CONTAINER_CLASS (toolbar_class);
 	container_class->remove = impl_remove;
@@ -1143,11 +1139,11 @@ bonobo_ui_toolbar_class_init (BonoboUIToolbarClass *toolbar_class)
 static void
 bonobo_ui_toolbar_instance_init (BonoboUIToolbar *toolbar)
 {
-	BonoboUIToolbarPrivate *priv;
+	AtkObject *ao;
 	BonoboUIToolbarStyle style;
+	BonoboUIToolbarPrivate *priv;
 
 	GTK_WIDGET_SET_FLAGS (toolbar, GTK_NO_WINDOW);
-	GTK_WIDGET_UNSET_FLAGS (toolbar, GTK_CAN_FOCUS);
 
 	priv = g_new (BonoboUIToolbarPrivate, 1);
 
@@ -1171,6 +1167,10 @@ bonobo_ui_toolbar_instance_init (BonoboUIToolbar *toolbar)
 	priv->tooltips                    = gtk_tooltips_new ();
 
 	toolbar->priv = priv;
+
+	ao = gtk_widget_get_accessible (GTK_WIDGET (toolbar));
+	if (ao)
+		atk_object_set_role (ao, ATK_ROLE_TOOL_BAR);
 }
 
 void
@@ -1306,12 +1306,16 @@ bonobo_ui_toolbar_insert (BonoboUIToolbar *toolbar,
 		G_CALLBACK (item_set_want_label_cb),
 		toolbar, 0);
 
+	g_object_ref (toolbar);
+	g_object_ref (item);
+
 	set_attributes_on_child (item, priv->orientation, priv->style);
 	parentize_widget (toolbar, GTK_WIDGET (item));
 
-	g_assert (GTK_WIDGET (item)->parent == GTK_WIDGET (toolbar));
-
 	gtk_widget_queue_resize (GTK_WIDGET (toolbar));
+
+	g_object_unref (item);
+	g_object_unref (toolbar);
 }
 
 GList *
